package structures;

/**
 * Maintains a heap of unique values.
 * @author Brian Bushnell
 * @date July 6, 2016
 *
 */
public class LongHeapSet {
	
	public LongHeapSet(int limit_){
		limit=limit_;
		heap=new LongHeap(limit, true);
		set=new LongHashSet(limit*2);
	}
	
	public boolean add(long value){
		assert(heap.size()==set.size());
		if(heap.hasRoom()){
			if(set.add(value)){
				heap.add(value);
				assert(heap.size()==set.size());
				return true;
			}
			assert(heap.size()==set.size());
			return false;
		}
		
		final long bottom=heap.peek();
		if(value>bottom){
			if(set.add(value)){
				set.remove(bottom);
				assert(set.size()<=limit);
				heap.add(value);
				assert(heap.size()<=limit);
				assert(heap.size()==set.size());
				return true;
			}
		}
		assert(heap.size()==set.size());
		return false;
	}
	
	protected void clear(){
		assert(heap.size()==set.size()) : heap.size()+", "+set.size();
		if(heap.size()<1){
			assert(set.size()<1) : heap.size()+", "+set.size();
			return;
		}
		heap.clear();
		set.clear();
		assert(heap.size()==set.size());
	}
	
	public void add(LongHeapSet b){
		assert(heap.size()==set.size());
		final long[] array=b.heap.array();
		final int blen=b.heap.size();
		for(int i=1; i<=blen; i++){
			add(array[i]);
		}
		assert(heap.size()==set.size());
	}
	
	public int size(){
		assert(heap.size()==set.size());
		return heap.size();
	}
	
	public int capacity(){return heap.capacity();}
	
	final int limit;
	public LongHeap heap;
	public LongHashSet set;
	
}
